module Feature.Generation.Passwords 
  ( Password
  , suggestedScheme
  , xkcd
  , oldschool
  , gibberish 
  ) where

import Core.Types
import Feature.Generation.Shared 
  ( camelCase
  , rCharSym
  , rIndex
  , validChars
  , validNumbers
  , validSymbols 
  )

import Data.List (singleton)

newtype Password = Password' String
  deriving (Eq, Ord)

instance Show Password where 
  show (Password' a) = a

suggestedScheme :: Int -> IO Password
suggestedScheme i 
  | i > 17    = xkcd
  | i > 12    = oldschool
  | otherwise = gibberish i

xkcd :: IO Password
xkcd = do
  wOne   <- rCamel
  wTwo   <- rCamel
  wThree <- rCamel
  wFour  <- rCamel
  return  $ Password' (wOne <> wTwo <> wThree <> wFour)

oldschool :: IO Password
oldschool = do
  wOne <- rCamel
  wTwo <- rCamel
  nOne <- rNum
  nTwo <- rNum 
  nThr <- rNum
  nFou <- rNum
  sOne <- rSym
  return 
    $ Password' (wOne <> wTwo 
   <> show nOne <> show nTwo <> show nThr <> show nFou
   <> pure sOne)

gibberish :: Int -> IO Password
gibberish i = go i (return "")
  where
    go :: Int -> IO String -> IO Password
    go 0 s = Password' <$> s
    go i s = go (i - 1) (s <> (singleton <$> rCharSym))

rNum :: IO Int
rNum = rIndex validNumbers

rWord :: IO String
rWord = wordList >>= rIndex

rSym :: IO Char
rSym = rIndex validSymbols

rCamel :: IO String
rCamel = camelCase <$> rWord

wordList :: IO [String]
wordList = fmap lines (readFile "./src/Feature/Generation/wordlist.txt")
